﻿using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;

namespace REG_MARK_LIB
{
    public class REG_MARK_LIB
    {
        /// <summary>
        /// Список регионов на Российских автомобильных номерах
        /// </summary>
        public static int[] REGIONS = { 190,
                                        177,
                                        199,
                                        977,
                                        102,
                                        103,
                                        277,
                                        297,
                                        299,
                                        111,
                                        113,
                                        116,
                                        118,
                                        121,
                                        122,
                                        125,
                                        126,
                                        130,
                                        134,
                                        136,
                                        142,
                                        147,
                                        750,
                                        152,
                                        154,
                                        155,
                                        156,
                                        158,
                                        161,
                                        163,
                                        164,
                                        172,
                                        173,
                                        174,
                                        197,
                                        777,
                                        180,
                                        181,
                                        777,
                                        184,
                                        185,
                                        186,
                                        702,
                                        716,
                                        123,
                                        725,
                                        138,
                                        150,
                                        790,
                                        252,
                                        754,
                                        159,
                                        761,
                                        763,
                                        166,
                                        774,
                                        797,
                                        178,
                                        193,
                                        124,
                                        196,
                                        799,
                                        198};

        /// <summary>
        /// Метод, для проверки номера на валидность
        /// </summary>
        /// <param name="mark">номер</param>
        /// <returns>true - строка является валидным авто номером, false - строка не является валидным авто номером</returns>
        public static bool CheckMark(string mark)
        {
            Regex regex = new Regex(@"^[a,b,e,k,m,h,o,p,c,t,y,x]{1}[0-9]{3}[a,b,e,k,m,h,o,p,c,t,y,x]{2}[0-9]{2,3}$", RegexOptions.IgnoreCase);
            if (!regex.IsMatch(mark)) return false;

            if (mark[6..mark.Length].Length == 2) return true;
            if (mark[6..mark.Length].Length == 3 && REGIONS.Where(i => i == Convert.ToInt32(mark[6..mark.Length])).FirstOrDefault() != default) return true;

            return false;
        }

        /// <summary>
        /// Метод, для получения следующего по порядку номера
        /// </summary>
        /// <param name="mark">номер, для которого будет осуществляться поиск</param>
        /// <returns>следующий по порядку номер</returns>
        /// <exception cref="ArgumentException">Возникает, когда строка аргумента не является валидным номером</exception>
        /// <exception cref="Exception">Возникает, когда следующий номер найти невозможно</exception>
        public static string GetNextMarkAfter(string mark)
        {
            if (!CheckMark(mark)) throw new ArgumentException();

            int[] code = ConvertMarkToInts(mark);

            if (code[1] == 999)
            {
                if (code[0] == 21)
                {
                    if (code[3] == 21)
                    {
                        if (code[2] == 21) throw new Exception("Следующего номера не существует!");
                        else
                        {
                            code[2] += 1;
                            code[3] = 10;
                            code[1] = 1;
                            code[0] = 10;
                        }
                    }
                    else
                    {
                        code[3] += 1;
                        code[1] = 1;
                        code[0] = 10;
                    }
                }
                else
                {
                    code[0] += 1;
                    code[1] = 1;
                }
            }
            else code[1] += 1;

            return ConvertIntsMarkToString(code);
        }

        /// <summary>
        /// Метод, для получения следующего по порядку номера, в рамках заданных границ
        /// </summary>
        /// <param name="prevMark">номер, для которого будет осуществляться поиск</param>
        /// <param name="rangeStart">нижняя граница поиска</param>
        /// <param name="rangeEnd">верхняя граница поиска</param>
        /// <returns>следующий по порядку номер</returns>
        /// <exception cref="ArgumentException">Возникает, когда строка аргумента не является валидным номером</exception>
        /// <exception cref="Exception">Возникает, когда следующий номер найти невозможно</exception>
        public static string GetNextMarkAfterInRange(string prevMark, string rangeStart, string rangeEnd)
        {
            if (!CheckMark(prevMark) || !CheckMark(rangeStart) || !CheckMark(rangeEnd)) throw new ArgumentException();

            if (CompareMarks(rangeStart,rangeEnd) == -1) return "out of stock";
            string nextMark = GetNextMarkAfter(prevMark);
            if (CompareMarks(rangeStart, prevMark) == -1 || CompareMarks(prevMark, rangeEnd) == -1) return "out of stock";
            return nextMark;
        }

        /// <summary>
        /// Метод, для поиска количества номеров, в заданных границах
        /// </summary>
        /// <param name="mark1">1 граница счета</param>
        /// <param name="mark2">2 граница счета</param>
        /// <returns>количество номеров в диапазоне, включая границы</returns>
        /// <exception cref="ArgumentException">Границы, не являются валидными автомобильными номерами</exception>
        public static int GetCombinationsCountInRange(string mark1, string mark2)
        {
            if (!CheckMark(mark1) || !CheckMark(mark2)) throw new ArgumentException();

            int counter = 1;
            string code1, code2;

            switch(CompareMarks(mark1, mark2))
            {
                case 1:
                    {
                        code1 = mark1;
                        code2 = mark2;
                        break;
                    }
                case -1:
                    {
                        code1 = mark2;
                        code2 = mark1;
                        break;
                    }
                default: return 1;
            }

            while(code1 != code2)
            {
                code1 = GetNextMarkAfter(code1);
                counter++;
            }

            return counter;
        }


        /// <summary>
        /// Метод, преобразовывающий номер в массив чисел
        /// </summary>
        /// <param name="mark">номер</param>
        /// <returns>массив чисел на основе номера</returns>
        private static int[] ConvertMarkToInts(string mark)
        {
            mark = mark.ToUpper();
            int[] result = new int[mark.Length - 2];
            Dictionary<char, int> keyValuePairs = new Dictionary<char, int> {{'A', 10},
                                                                             {'B', 11},
                                                                             {'E', 12},
                                                                             {'K', 13},
                                                                             {'M', 14},
                                                                             {'H', 15},
                                                                             {'O', 16},
                                                                             {'P', 17},
                                                                             {'C', 18},
                                                                             {'T', 19},
                                                                             {'Y', 20},
                                                                             {'X', 21},
                                                                             {'0', 0},
                                                                             {'1', 1},
                                                                             {'2', 2},
                                                                             {'3', 3},
                                                                             {'4', 4},
                                                                             {'5', 5},
                                                                             {'6', 6},
                                                                             {'7', 7},
                                                                             {'8', 8},
                                                                             {'9', 9}};
            result[0] = keyValuePairs[mark[0]];
            result[1] = Convert.ToInt32(mark[1..4]);
            for (int i = 4; i < mark.Length; i++) result[i - 2] = keyValuePairs[mark[i]];
            return result;
        }

        /// <summary>
        /// Метод, преобразующий массив на основе номера в текстовый вариант номера
        /// </summary>
        /// <param name="mark">массив на основе номера</param>
        /// <returns>номер в текстовой форме</returns>
        private static string ConvertIntsMarkToString(int[] mark)
        {
            StringBuilder markString = new StringBuilder();
            Dictionary<int, char> keyValuePairs = new Dictionary<int, char> {{10, 'A'},
                                                                             {11 , 'B'},
                                                                             {12 , 'E'},
                                                                             {13 , 'K'},
                                                                             {14 , 'M'},
                                                                             {15 , 'H'},
                                                                             {16 , 'O'},
                                                                             {17 , 'P'},
                                                                             {18 , 'C'},
                                                                             {19 , 'T'},
                                                                             {20 , 'Y'},
                                                                             {21 , 'X'},
                                                                             {0 , '0'},
                                                                             {1 , '1'},
                                                                             {2 , '2'},
                                                                             {3 , '3'},
                                                                             {4 , '4'},
                                                                             {5 , '5'},
                                                                             {6 , '6'},
                                                                             {7 , '7'},
                                                                             {8 , '8'},
                                                                             {9 , '9'}};
            markString.Append(keyValuePairs[mark[0]]);
            StringBuilder sb = new StringBuilder(mark[1].ToString());
            while (sb.Length < 3)
            {
                sb = new StringBuilder("0" + sb.ToString());
            }
            markString.Append(sb.ToString());
            for (int i = 2; i < mark.Length; i++) markString.Append(keyValuePairs[mark[i]]);
            return markString.ToString();
        }

        /// <summary>
        /// Метод, для сравнения двух номеров в соответствии с установленным порядком
        /// </summary>
        /// <param name="strForComp1">Номер для сравнения №1</param>
        /// <param name="strForComp2">Номер для сравнения №2</param>
        /// <returns>1 - номер №2 больше; -1 - номер №1 больше; 0 - номера одинаковы</returns>
        private static int CompareMarks(string strForComp1, string strForComp2)
        {
            int[] code1 = ConvertMarkToInts(strForComp1);
            int[] code2 = ConvertMarkToInts(strForComp2);
            if (code2[2] > code1[2]) return 1;
            else if (code2[2] < code1[2]) return -1;
            else
            {
                if (code2[3] > code1[3]) return 1;
                else if (code2[3] < code1[3]) return -1;
                else
                {
                    if (code2[0] > code1[0]) return 1;
                    else if (code2[0] < code1[0]) return -1;
                    else
                    {
                        if (code2[1] > code1[1]) return 1;
                        else if (code2[1] < code1[1]) return -1;
                        else return 0;
                    }
                }
            }
        }
    }
}